@@ -258,6 +258,16 @@ impl Pacer {
         self.sender.max_bandwidth()
     }
 
+    #[cfg(feature = "qlog")]
+    pub fn send_rate(&self) -> Option<Bandwidth> {
+        self.sender.send_rate()
+    }
+
+    #[cfg(feature = "qlog")]
+    pub fn ack_rate(&self) -> Option<Bandwidth> {
+        self.sender.ack_rate()
+    }
+
     pub fn on_app_limited(&mut self, bytes_in_flight: usize) {
         self.pacing_limited = false;
         self.sender.on_app_limited(bytes_in_flight);
