@@ -447,6 +447,7 @@ pub struct GRecovery {
     pub bytes_lost: u64,
 
     max_datagram_size: usize,
+    time_sent_set_to_now: bool,
 
     #[cfg(feature = "qlog")]
     qlog_metrics: QlogMetrics,
@@ -468,6 +469,16 @@ pub struct GRecovery {
 }
 
 impl GRecovery {
+    #[cfg(feature = "qlog")]
+    fn send_rate(&self) -> Bandwidth {
+        self.pacer.send_rate().unwrap_or(Bandwidth::zero())
+    }
+
+    #[cfg(feature = "qlog")]
+    fn ack_rate(&self) -> Bandwidth {
+        self.pacer.ack_rate().unwrap_or(Bandwidth::zero())
+    }
+
     pub fn new(recovery_config: &RecoveryConfig) -> Option<Self> {
         let cc = match recovery_config.cc_algorithm {
             CongestionControlAlgorithm::Bbr2Gcongestion => BBRv2::new(
@@ -499,6 +510,7 @@ impl GRecovery {
             bytes_lost: 0,
 
             max_datagram_size: recovery_config.max_send_udp_payload_size,
+            time_sent_set_to_now: cc.time_sent_set_to_now(),
 
             #[cfg(feature = "qlog")]
             qlog_metrics: QlogMetrics::default(),
@@ -688,7 +700,11 @@ impl RecoveryOps for GRecovery {
         &mut self, pkt: Sent, epoch: packet::Epoch,
         handshake_status: HandshakeStatus, now: Instant, trace_id: &str,
     ) {
-        let time_sent = self.get_next_release_time().time(now).unwrap_or(now);
+        let time_sent = if self.time_sent_set_to_now {
+            now
+        } else {
+            self.get_next_release_time().time(now).unwrap_or(now)
+        };
 
         let epoch = &mut self.epochs[epoch];
 
@@ -1133,7 +1149,15 @@ impl RecoveryOps for GRecovery {
             cwnd: self.cwnd() as u64,
             bytes_in_flight: self.bytes_in_flight.get() as u64,
             ssthresh: self.pacer.ssthresh(),
-            pacing_rate: self.delivery_rate().to_bytes_per_second(),
+
+            pacing_rate: Some(
+                self.pacer
+                    .pacing_rate(self.bytes_in_flight.get(), &self.rtt_stats)
+                    .to_bytes_per_second(),
+            ),
+            delivery_rate: Some(self.delivery_rate().to_bytes_per_second()),
+            send_rate: Some(self.send_rate().to_bytes_per_second()),
+            ack_rate: Some(self.ack_rate().to_bytes_per_second()),
         };
 
         self.qlog_metrics.maybe_update(qlog_metrics)
